

/*
    This code runs in real time.  It does not use the delay() function.
    It is intended to be a simple project for beginners to the Arduino. I
    use an UNO.

    It is an example.  This sketch reads a thermometer chip and reports the
	temp once/minute.
	It can tell the difference betweena TMP36 and an LM35 assuming the
	temp is more or less room temp.  Ajustment will be needed for temps outside
	of that range.

	Either chip can be wired to any analog Ardiono input pin.

    It also calls my blinkenlights function.  That is the point.

    This code is free to use/modify for your entertaiment.

    TR 3/11/2016
*/

#define TRUE 1
#define FALSE 0

#define INPIN A2 //pick a pin !This goes to the center pin of the sensor, the other 2 
// go to Vcc (+5V) and GND on the Arduino.

// output  "DISPLAY" conflicts witha system #define hence DISPLAYVAL
#define DISPLAYVAL(arg) Serial.print(#arg "  = "); Serial.println(arg)

// detect LM35 v TMP36
#define DAMNCOLD 400 // to detect LM35 v. TMP36, assumes close to room temp.
#define ADJUST 450 // should be 500 mv, but 450 is closer to what works

// the scaling factor for INTERNAL voltage reference
#define SCALE (1.1/1024.0) //PJS says this could be better, don't understand it.

// globals used for timing control
// I switched to using static variables in the thermometer and blinkenlights functions.
const unsigned long howlong = 60000UL;// 1 minute in milliseconds


void setup() {

  Serial.begin(9600); //for reporting the temperature
  analogReference(INTERNAL);// sets the refernce voltage to 1.1V

  // pick any three digital pins, I avoid 0 & 1 because of serial IO
  pinMode(2, OUTPUT);// Arduino digital output pins
  pinMode(3, OUTPUT);// used to drive das blinkenlights
  pinMode(4, OUTPUT);


}// end of setup()


void loop() {
  thermometer(); // prints the temp once every minute
  blinkenlights(); // provides a (tiny) bit of entertainment



  // plenty of time to do something interesting here
  // If it takes < 500 ms it won't even disturb the light show.
  // If it take longer, you can adjust the constant in blinkenlights()
  // to keep the display moving smoothly, although more slowly.

}//end of loop()


void thermometer() {
  unsigned long thistime;
  unsigned long timepassed;

  static unsigned long th_lasttime = 0;
  static float lowest = 100.00;
  static float highest = 0.00;



  // decide if enough time has passed to read the temp, I read it once/minute.
  // Adjust the const variable "howlong" to change the reporting interval.



  thistime = millis();

  timepassed = thistime - th_lasttime;

  if ((timepassed) < howlong) {// 1 minute, does not check for overflow
    return;
  } else {
    th_lasttime = thistime;

    // the TMP36 has a slope of 10 mV/°C.  25°C is 750 mV.
    // the LM35 has the same slope but 25°C is 250 mV
    // I can't explain ADJUST not being 500 mV
    float rawvolts = analogRead(INPIN) ;
    //DISPLAYVAL(rawvolts) ;

    // decide if we have a TMP36 or an LM35, assumes more or less room temp.
    // both chips have the same slope, different Y axis intercept
    if (rawvolts < DAMNCOLD) rawvolts += ADJUST;

    float scaledvolts = rawvolts * SCALE;
    //DISPLAYVAL(scaledvolts);


    float millivolts = scaledvolts * 1000.0;//* convert to millivolts
    //DISPLAYVAL(millivolts) ;


    //  covert fron mV to degrees C -500 converts 750mV at 25C to 250mV
    float ctemp = (millivolts - 500 ) / 10.0;
    //DISPLAYVAL(ctemp) ;

    // standard C to F conversion
    float fahrenheittemp = (ctemp * (9.0 / 5.0)) + 32.0;
    DISPLAYVAL(fahrenheittemp);


    if (lowest > fahrenheittemp) {
      lowest = fahrenheittemp;
    }
    if (highest < fahrenheittemp) {
      highest = fahrenheittemp;
    }
    DISPLAYVAL(lowest) ;
    DISPLAYVAL(highest);


    Serial.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
    Serial.println();
  }

}//end of thermometer


/*
     This is a simple sketch to provide am interrupt safe delay timer and
  some mildly amusing blinking lights.  It does not use delay().


     This code uses three 3 digital IO lines.  Awfully expensive for very
  little benefit, unless you just love blinklights.

  I use this code to drive a 74HC238N 3 to 8 decoder..

     Call this routine in your loop() routine.  It will take very little time.
*/
#define NUMLEDS 8

// start at zero, on subsequent iterations, up will start at 1,
// while down will go to 0. This keeps it from hovering at the ends of
// the blinkenlights.
int up = 0;
int down = NUMLEDS;

int thisled, nextled;

// this code drives the 3 to 8 decoder chip (74HC238)
void _advance() {
// This code drives the 238 chip.
  if (up < NUMLEDS) {

    digitalWrite(2, (up & 0x01) ? HIGH : LOW);
    digitalWrite(3, (up & 0x02) ? HIGH : LOW);
    digitalWrite(4, (up & 0x04) ? HIGH : LOW);
    up ++;

  } else {
    
    down--;
    digitalWrite(2, (down & 0x01) ? HIGH : LOW);
    digitalWrite(3, (down & 0x02) ? HIGH : LOW);
    digitalWrite(4, (down & 0x04) ? HIGH : LOW);

    if (down == 0) {
      // 1 and not zero to avoid hovering
      down = NUMLEDS - 1;
      up = 1;
    }
  }
}//end of _advance()

void blinkenlights() {

  static unsigned long bl_lasttime = 0;
  unsigned long thistime;


  thistime = millis();

// adjust 100 to taste...
  if (thistime - bl_lasttime > 100) {
    bl_lasttime = thistime;
    _advance();

  }

}// end of blinkenlights





